"use strict";
const { memsourceLib, memsourceAPI } = require("../../../memsource");
const Storage = require("../../../storage");
const ProjectActions = require("../projectActions");
const utils = require("../../utils");
const config = require("../../config.json");

function checkProjectAnalysis(project) {
  console.log(project.uid);
  let authenticationToken;
  return utils
    .getAuthenticationToken(config.userMemSource, config.passMemSource)
    .then(token => (authenticationToken = token))
    .then(() => memsourceLib.getLatestProjectAnalysis(authenticationToken, project.uid))
    .then(res => {
      let isCompleted = true;

      res.analyseLanguageParts.forEach(langPart => {
        if (!langPart.data.available) isCompleted = isCompleted && false;
      });

      if (res.id && isCompleted) {
        res.ProjectUid = project.uid;
        return Storage.saveAnalyses([res]).then(() => Storage.updateProjectProcessStatus(project.uid, "READY"));
      }

      // Check if analysis was requested 60 min ago. If so, create a new one.
      const diffInMilis = Math.abs(project.analysisRequestedDate - new Date());

      if (Math.floor(diffInMilis / 1000 / 60) > 60) {
        console.log(`Requesting a new analysis for project ${project.uid}`);
        return ProjectActions.requestAnalysis(authenticationToken, project, project.Jobs);
      }
      return Promise.resolve();
    });
}

function checkAnalysis() {
  return Storage.initConnection()
    .then(() =>
      Storage.getAllProjectsByStatus("ANALYSING").then(projects =>
        Promise.all(projects.map(project => checkProjectAnalysis(project)))
      )
    )
    .then(() => Storage.closeConnection());
}

module.exports = {
  checkProjectAnalysis,
  checkAnalysis,
};
