"use strict";
const { memsourceLib, memsourceAPI } = require("../../../memsource");
const utils = require("../../utils");
const config = require("../../config.json");
const util = require("../../utils/util.js");
const fs = require("fs");

const PM_USER_EMAIL = "mario.bogarin@welocalize.com";
const PM_USER_ID = 3;

let currentPath = process.cwd();
let lambdasPath = currentPath.replace("controller", "lambdas");
const myFile = `${lambdasPath}/memFiles/test2.docx`;

function activeMemUser(user) {
  let authenticationToken;
  return utils
    .getAuthenticationToken(config.userMemSourceTest, config.passMemSourceTest)
    .then(token => (authenticationToken = token))
    .then(() => {
      console.log(`Activating mem user ${user}`);
      return memsourceLib.activateUser(authenticationToken, user);
    });
}

function desactivateMemUser(user) {
  let authenticationToken;
  return utils
    .getAuthenticationToken(config.userMemSourceTest, config.passMemSourceTest)
    .then(token => (authenticationToken = token))
    .then(() => {
      console.log(`Inactivating mem user ${user}`);
      return memsourceLib.deactivateUser(authenticationToken, user);
    });
}

function grantAcces(resource) {
  let authenticationToken;
  return utils
    .getAuthenticationToken(config.userMemSourceTest, config.passMemSourceTest)
    .then(token => (authenticationToken = token))
    .then(() => {
      //return memsourceAPI;
      console.log(`Granting access to mem user ${user}`);
      //1 - grant access to mem
    });
}

function checkMemProjectStatus(task) {
  let authenticationToken;
  let uidProject;
  let uidJob;
  let sdkCon;

  //- If the job it's completed  -> We should fulfill the task (Translation or copy-edit)
  //- The memSource projects it's completed -> Whe should fulfill the delivery task in pantheon

  return util
    .initSdk(PM_USER_EMAIL)
    .then(sdk => {
      sdkCon = sdk;
      return sdk.projectIndex({ id: task[0].projectId });
    })
    .then(project => {
      console.log(project);
      uidProject = project.data[0].meta.uidProject;
      return sdkCon.clientServiceGroupShow(task[0].clientServiceItemId); //task.clientServiceItemId
    })
    .then(csi => {
      uidJob = csi.meta.uidJob;
      return utils.getAuthenticationToken(config.userMemSourceTest, config.passMemSourceTest);
    })
    .then(token => (authenticationToken = token))
    .then(() => {
      console.log(`Token received ${authenticationToken}`);

      console.log(`Checking mem job status ${uidJob}`);
      mesourceLib
        .areAllJobsComplete(authenticationToken, uidProject, uidJob.map(obj => obj.uid))
        .then(allJobsComplete => {
          console.log(allJobsComplete);
          if (allJobsComplete) {
            //Promise.all(task.map(t => sdkCon.taskFulfill(t.id, PM_USER_ID))).then(res => {
            //console.log(res);
            //});
          }
        });

      console.log(`Checking mem project status ${uidProject}`);
      //if mem project is completed - fulfill delivery task
      //return memsourceAPI;
      return uidProject;
    })
    .then(uidProject => {
      //If res == completed - update pantheon status to completed
      const query = { projectId: projectId, state: "completed" };
      console.log(query);
      return sdkCon.projectUpdate(query);
    });
}

function cloneMemProjects(templateData) {
  let projectUid;
  let authenticationToken;
  const jobs = [];
  return utils
    .getAuthenticationToken(config.userMemSource, config.passMemSource)
    .then(token => (authenticationToken = token))
    .then(() => {
      console.log(config.userMemSource + " " + config.passMemSource);
      console.log("myToken", authenticationToken);
      return memsourceAPI.cloneProject(authenticationToken, config.memProjectTemplateUid, {
        name: `${config.memProjectAutoName}_${new Date().toISOString()}`,
      });
    })
    .then(res => {
      projectUid = res.uid;
      console.log("myProjectUid", projectUid);
      const formData = {
        file: fs.createReadStream(myFile),
      };
      const languages = res.targetLangs;
      return Promise.all(
        languages.map(lang =>
          memsourceAPI.createJob(authenticationToken, projectUid, lang, formData).then(job => jobs.push(job))
        )
      );
    })
    .then(() =>
      Promise.all(
        jobs.map(job => {
          const jobUid = job.jobParts[0].uid;
          console.log("myJobUid", jobUid);
          return memsourceAPI.setJobStatus(authenticationToken, projectUid, jobUid, "Completed");
        })
      )
    )
    .then(res => console.log(res))
    .catch(err => {
      console.log(err);
    });
}

//Once the translation/copy edit task will be ready - Active MemSource user and asign the job
function assignProcess(taskId) {
  console.log("llego");
  const vendor = 73;
  const taskCsi = 94;
  const data = {};
  let sdkCon;
  return util
    .initSdk(PM_USER_EMAIL)
    .then(sdk => {
      sdkCon = sdk;
      const query = { id: taskId };
      return sdk.taskIndex(query);
    })
    .then(task => {
      console.log("myTask", task);
      //let vendor = task[0].assignedVendorId;
      return sdkCon.userShow(vendor);
    })
    .then(user => {
      data.mailUser = user.data.googleId;
      return sdkCon.clientServiceGroupShow(taskCsi); //task.clientServiceItemId
    })
    .then(csi => {
      data.uidJob = csi.meta.uidJob;
      return data;
    })
    .catch(err => {
      console.log("Error: ", err);
    });
}

module.exports = {
  activeMemUser,
  desactivateMemUser,
  grantAcces,
  checkMemProjectStatus,
  cloneMemProjects,
  assignProcess,
};
