"use strict";

const fetch = require("node-fetch");
const sdk = require("@welocalize/pantheon-api-sdk-node6");

const AWS = require("aws-sdk");
const lambda = new AWS.Lambda({
  region: "us-west-2",
});

// Determine the environment. Needed to init the SDK properly.
let SDK_ENV = process.env.NODE_ENV;

SDK_ENV = "local"; //Forced

if (SDK_ENV === "dev") SDK_ENV = "development";

// -->BEGIN PRIVATE FUNCTIONS<--

// Invokes the Lambda that creates a JWT.
function _invokeCreateJwt(payload) {
  //debugger;
  return new Promise(function(resolve, reject) {
    const params = {
      FunctionName: "tekmor-v3-dev-createJwt",
      InvocationType: "RequestResponse",
      Payload: JSON.stringify(payload),
    };
    //debugger;
    return lambda
      .invoke(params)
      .promise()
      .then(res => {
        //debugger;
        return resolve(JSON.parse(res.Payload).body);
      })
      .catch(err => {
        //debugger;
        logr.error(`Error in Task Manager util.js/_invokeCreateJwt(): ${err}`, err, META);
      });
  });
}

// -->END PRIVATE FUNCTIONS<--

// -->BEGIN PUBLIC FUNCTIONS<--

// Retrieves a JWT based on the User ID.
async function getJwt(oktaEmail) {
  const payload = {
    //principal: userId,
    target: "okta",
    id: oktaEmail,
  };

  return _invokeCreateJwt(payload);
}

// Initialize the sdk.
async function initSdk(oktaEmail) {
  try {
    // Get the JWT.
    const val = await getJwt(oktaEmail);
    let jwtval = JSON.parse(val);
    jwtval = jwtval.data;

    // Initialize the SDK.
    await sdk.init({
      env: SDK_ENV,
      jwt: jwtval,
      apiKey: "...",
      fetch,
      trace: false,
    });

    console.log("SDK initialized.");
    return sdk;
  } catch (err) {
    throw err;
  }
}

// This function returns a Promise the resolves once the child process completes.
function waitForChildProcess(childProcess) {
  return new Promise(function(resolve, reject) {
    childProcess.on("close", function(code) {
      console.log(`child process has completed with code ${code}`);

      if (code === 0) return resolve(true);
      return resolve(false);
    });
  });
}

// -->END PUBLIC FUNCTIONS<--

module.exports.getJwt = getJwt;
module.exports.initSdk = initSdk;
module.exports.waitForChildProcess = waitForChildProcess;
